/*--------------------------------------------------------------
 * mapgen.c    - translate map tables
 *
 * usage: mapgen file ...
 *
 * Autor: Frank Meyer
 *--------------------------------------------------------------
 */

#include <stdio.h>

#include "map-table.h"             /* data structures        */

#ifndef TRUE
# define TRUE    (1)
# define FALSE    (0)
#endif


int
main (argc, argv)
int argc;
char * argv[];
{
    FILE * fp_in;            /* file pointer for read        */
    FILE * fp_out;          /* file pointer for write        */
    char fname[128];              /* new filename            */
    int length;                 /* length of filename        */
    int rtc;                 /* return code of translate()   */
    int i;                         /* counter                */

    if (argc < 2)
    {
    fprintf (stderr, "usage: %s file ...\n", argv[0]);
    exit (1);
    }

    for (i = 1; i < argc; i++)   /* loop over all files ...  */
    {
    strcpy (fname, argv[i]);  /* check correct suffix        */

    length = strlen (fname);

    if (length < 4 || strcmp (fname + length - 3, ".IN"))
    {
     fprintf (stderr, "%s: %s has no suffix '.IN' !\n",
              argv[0], fname);
     exit (1);
    }

     fp_in = fopen (fname, "r");              /* open file  */

    if (! fp_in)                           /* successful ?  */
    {                                          /* no !      */
     fprintf (stderr, "%s: cannot open %s !\n",
              argv[0], fname);
     exit (1);
    }

    strcpy (fname + length - 3, ".OUT"); /* xx.IN -> xx.OUT */

    fp_out = fopen (fname, "w");        /* open output file */

    if (! fp_out)
    {                                      /* open error !  */
     fprintf (stderr, "%s: cannot open %s !\n",
              argv[0], fname);
     fclose (fp_in);
     exit (1);
    }

     rtc = translate (fp_in, fp_out);   /* translate table  */

    (void) fclose (fp_in);              /* close the files  */
    (void) fclose (fp_out);

    if (rtc != 0)
    {
     exit (1);                          /* error occured !  */
    }
    printf ("%s --> %s\n", argv[i], fname); /* information  */
    }

 return (0);
} /* main (argc, argv[]) */


int
translate (fp_in, fp_out)
FILE * fp_in;
FILE * fp_out;
{
    char buf[256];                        /* input buffer  */
    int i, j;                                 /* counters  */
    char esc_sequence[64];             /* input sequence   */
    char * p;                          /* ptr esc_squence  */
    char * chp;                        /* ptr charset[]    */
    int hex_code;                             /* hex code  */
    int chset_no;                      /* character set no */
    int iso_char;                      /* iso character    */
    int ch_out;                        /* output character */
    int in_first_part = TRUE;          /* flag: 1st or 2nd */

    for (i = 0; i < MAX_USED_CHARSETS; i++)
                               /* 1st clear charset array! */
    {                          /* because this function    */
    for (j = 0; j < MAX_ESC_LEN; j++)
                                  /* could be called more  */
    {                                    /* than once      */
  charset[i][j] = '\0';
    }
    }

    for (i = 0; i < 256; i++)     /* clear also map_table  */
    {
    map_table[i].chset_no = 0; /* default: ASCII chset #0  */
    map_table[i].ch_out = i;   /* default: no translation  */
    }

    while (fgets (buf, 255, fp_in)) /* read line by line   */
    {
    if (*buf != '#' && *buf != '\n')
                               /* Comment or empty line ?  */
    {                                       /* no ...      */
  if (in_first_part)           /* in 1st part of file ?    */
  {                                  /* yes                */
   if (! strncmp (buf, "END", 3))    /* end of 1st part ?  */
        {                              /* yes, reset flag  */
      in_first_part = FALSE;
        }
        else                   /* read charset number and  */
        {                              /* escape sequence  */
         if (sscanf (buf, "%d %s", &chset_no,
             esc_sequence) != 2)
           {
            fprintf (stderr, "Read error.\n");
            return (-1);
           }
      if (chset_no >= MAX_USED_CHARSETS)
      {
        fprintf (stderr, "Illegal character set number.\n");
        return (-1);
      }
      chp = charset[chset_no]; /* use a ptr to fill array  */

      for (p = esc_sequence; *p; p += 2)
      {
       if (sscanf (p, "%2x", &hex_code) != 1)
          {
           fprintf (stderr, "Read error.\n");
           return (-1);
          }
           *chp++ = hex_code;  /* store hex code binary    */
      }
      *chp = '\0';           /* terminate charset[]        */
     }
  }
  else                          /* the 2nd part ...        */
  {                           /* read three columns        */
   if (sscanf (buf, "%x %d %x",
          &iso_char, &chset_no, &ch_out) != 3)
        {
         fprintf (stderr, "Read error.\n");
         return (-1);
        }
        map_table[iso_char].chset_no = chset_no;
        map_table[iso_char].ch_out = ch_out;
  }
 }
}

 fwrite (charset, sizeof (charset), 1, fp_out);
 fwrite (map_table, sizeof (map_table), 1, fp_out);
                                       /* write map_table  */

    return (0);                        /* return: success  */
} /* translate (fp_in, fp_out) */


/*-----------------------------------------------------------
 * End of mapgen.c
 *-----------------------------------------------------------
 */
